/**
 * Copyright (c) 2000-2011 ActiveState Software Inc.
 * See the file LICENSE.txt for licensing information.
 */
if (!('extensions' in ko)) ko.extensions = {};
if (!('stackato' in ko.extensions)) ko.extensions.stackato = {};

(function() {
var prefs = ko.prefs;
// This is a list of 1 because it used to contain tclsh info as well.
["stackato.location"].forEach(function(prefName) {
    if (!prefs.hasPref(prefName)) {
        prefs.setStringPref(prefName, "");
    }
});
var bundle = Components.classes["@mozilla.org/intl/stringbundle;1"]
    .getService(Components.interfaces.nsIStringBundleService)
    .createBundle("chrome://stackatotools/locale/stackato.properties");
this.data = {};

this.getStackatoEnv = function(obj) {
    var prefs = ko.prefs;
    try {
        obj.stackato = prefs.getStringPref("stackato.location");
    } catch(ex) {
        dump("getting stackato: " + ex + "\n");
        obj.stackato = null;
    }
    if (!obj.stackato) {
        var prompt = bundle.GetStringFromName("Komodo needs to know where the Stackato executable is located");
        var title = bundle.GetStringFromName("Stackato Configuration");
        var defaultResponse = bundle.GetStringFromName("No");
        var response = ko.dialogs.yesNo(prompt, defaultResponse, null, title);
        if (response == bundle.GetStringFromName('Yes')) {
            prefs_doGlobalPrefs('stackatoItem');
            return;
        }
    }
    var probCharsPtn = /\s/;
    if (probCharsPtn.test(obj.stackato)) {
        obj.stackato = '"' + obj.stackato + '"';
    }
    var sysUtilsSvc = Components.classes['@activestate.com/koSysUtils;1'].getService(Components.interfaces.koISysUtils);
    var osSvc = Components.classes["@activestate.com/koOs;1"].getService(Components.interfaces.koIOs);
    var osPathSvc = Components.classes["@activestate.com/koOsPath;1"].getService(Components.interfaces.koIOsPath);
    var environSvc = Components.classes["@activestate.com/koUserEnviron;1"].getService(Components.interfaces.koIUserEnviron);
    var path = environSvc.get("PATH");
    obj.path = [osPathSvc.dirname(obj.stackato), path].join(osSvc.pathsep);
    try {
        var file = ko.views.manager.currentView.koDoc.file;
        if (file.scheme == "file") {
            obj.cwd = file.dirName;
        } else {
            obj.cwd = null;
        }
    } catch(ex2) {
        dump("error getting current view dir: " + ex2 + "\n");
    }
};

this.getRecentApp = function() {
    return this.getRecentFeature("appName");
};

this.setRecentApp = function(appName) {
    this.setRecentFeature("appName", appName);
};

this.getRecentFeature = function(name) {
    if (name in this.data) {
        var appObj = this.data[name];
        if (new Date().valueOf() - appObj.timestamp < 3600 * 1000) {
            return appObj.value;
        }
    }
    return "";
};

this.strip = function(s) {
    return s.replace(/^\s*(.*?)\s*$/, "$1");
};

this.getServicesInfo = function(envObj) {
    var koRunSvc = Components.classes["@activestate.com/koRunService;1"].getService(Components.interfaces.koIRunService);
    var command = envObj.stackato + " services";
    var env = "PATH=" + envObj.path;
    var output = {}, errors = {};
    var status = koRunSvc.RunAndCaptureOutput(command, null, env, null, output, errors);
    output = output.value;
    errors = errors.value;
    if (status || errors) {
        ko.dialogs.alert(bundle.formatStringFromName("stackato apps failed_status_X_errors", [status, errors], 2));
        return [];
    }
    var lines = output.split(/\r?\n/);
    var i = 0, len = lines.length;
    var systemServices = [];
    var provisionedServices = [];
    var separator_re = /^[\|\-]+$/;
    var fields2_re = /\|\s*(.*?)\s*\|\s*(.*?)\s*\|/;
    var fields3_re = /\|\s*(.*?)\s*\|\s*(.*?)\s*\|\s*(.*?)\s*\|/;
    var serviceVersionDescHeading = /\|\s*Service\s*\|\s*Version\s*\|\s*Description\s*\|/;
    var nameServiceHeading = /\|\s*Name\s*\|\s*Service\s*\|/;
    var retVal = {
        systemServices: {},
        provisionedServices: {}
    };
    var result;
    while (i < len) {
        if (serviceVersionDescHeading.test(lines[i])) break;
        i++;
    }
    if (i == len) {
        dump("Can't parse services output (no serviceVersionDescHeading)\n");
        return retVal;
    }
    var info;
    var line, field = "systemServices";
    for (; i < len; ++i) {
        line = lines[i];
        if (separator_re.test(line)) continue;
        else if (serviceVersionDescHeading.test(line)) continue;
        else if (nameServiceHeading.test(line)) {
            field = "provisionedServices";
        } else if ((result = fields3_re.exec(line)) !== null && field == "systemServices") {
            retVal.systemServices[this.strip(result[1])] = {
                version: this.strip(result[2]),
                description: this.strip(result[3])
            };
        }
        else if ((result = fields2_re.exec(line)) !== null && field == "provisionedServices") {
            retVal.provisionedServices[this.strip(result[1])] = {
                service: this.strip(result[2])
            };
        }
    }
    return retVal;
};

this.setRecentFeature = function(name, value) {
    if (!name in this.data || !this.data[name]) {
        this.data[name] = {};
    }
    var appObj = this.data[name];
    appObj.value = value;
    appObj.timestamp = new Date().valueOf();
};

this.getCurrentAppInfo = function(envObj) {
    var koRunSvc = Components.classes["@activestate.com/koRunService;1"].getService(Components.interfaces.koIRunService);
    var command = envObj.stackato + " apps";
    var env = "PATH=" + envObj.path;
    var output = {}, errors = {};
    var status = koRunSvc.RunAndCaptureOutput(command, null, env, null, output, errors);
    output = output.value;
    errors = errors.value;
    if (status ) {
        ko.dialogs.alert(bundle.formatStringFromName("stackato apps failed_status_X_errors", [status, errors], 2));
        return [];
    }
    var appInfoItems = [];
    dump("getCurrentAppInfo: Matching " + output + "\n");
    var pattern = /\|\s*(\S+?)\s*\|\s*(\d+)\s*\|\s*(\S+)\s*\|\s*(.*?)\s*\|\s*(.*?)\s*\|/g;
    //              | app name    | count      | health     | uris       | services    | 
    var result;
    while ((result = pattern.exec(output)) !== null) {
        dump("getCurrentAppInfo: got appName " + result[1] + "\n");
        var urls = result[4].split(/\s*,\s*/);
        if (urls.length == 1 && !urls[0]) urls = [];
        appInfoItems.push({
            name: result[1],
            count: result[2],
            health: result[3],
            urls: urls,
            services: result[5]
        });
        var item = appInfoItems[appInfoItems.length - 1];
        for (var p in item) {
            dump(p + ": " + item[p] + "\n");
        }
    }
    return appInfoItems;
};

this.getCurrentAppNames = function(envObj) {
    return this.getCurrentAppInfo(envObj).map(function(item) item.name);
};

this.getSelectedApp = function(envObj, appNames) {
    if (typeof(appNames) === "undefined") {
        appNames = this.getCurrentAppNames(envObj);
    }
    if (appNames.length == 0) {
        ko.dialogs.alert(bundle.GetStringFromName("No apps have been defined"));
        this.setRecentApp(null);
        return null;
    } else if (appNames.length == 1) {
        this.setRecentApp(appNames[0]);
        return appNames[0];
    } else {
        var currentAppName = this.getRecentApp();
        var idx = currentAppName ? appNames.indexOf(currentAppName) : -1;
        var title = bundle.GetStringFromName("Select an app");
        var prompt = bundle.GetStringFromName("Select the app to run");
        var selectionCondition = "one";
        var selectedIndex = idx === -1 ? 0 : idx;
        var newAppName = ko.dialogs.selectFromList(title, prompt, appNames, selectionCondition,
                                                   null, null, null, null, selectedIndex);
        if (newAppName !== null) {
            newAppName = newAppName[0];
            this.setRecentApp(newAppName);
        }
        return newAppName;
    }
};

this.doApplicationCommand = function(operation) {
    var obj = {};
    this.getStackatoEnv(obj);
    if (!obj.stackato || !obj.path) {
        dump("Bailing out of " + operation + "\n");
        return;
    }
    var appName = this.getSelectedApp(obj);
    if (!appName) {
        return;
    }
    var command = [obj.stackato, operation, appName].join(" ");
    var env = "PATH=" + obj.path;
    ko.run.runCommand(window, command, null, env,
                      false, false, false, "command-output-window");
};

this.doInformationCommand = function(operation) {
    var obj = {};
    this.getStackatoEnv(obj);
    if (!obj.stackato || !obj.path) {
        dump("Bailing out of " + operation + "\n");
        return;
    }
    var command = [obj.stackato, operation].join(" ");
    var env = "PATH=" + obj.path;
    ko.run.runCommand(window, command, null, env,
                      false, false, false, "command-output-window");
};
    
}).apply(ko.extensions.stackato);
